<?php


/**
 * Class to encapsulate a single LDAP Server configuration
 *
 */

class ldap_api_server {

  public $connection = NULL;
  public $server_addr = NULL;
  public $name;
  public $port = 389;
  public $tls = FALSE;
  public $sid;
  public $pwd_encryption;
  public $binddn;
  public $bindpw;
  public $basedns;  // array of basedns
  public $basedn; // set of basedns separated by returns
  public $user_attr;
  public $mail_attr = "mail";
  public $ldap_to_drupal_user;
  public $testing_drupal_username;
  public $module_data;
  
  /**
 * Constructor Method
 *
 * @param $params
 *   Can be:
 *     - an associative array of property-name => property-values
 *     - the id (sid) of an ldap server configuration stored by ldap_server module
 */
  function __construct($params = NULL) {
    $constructor_properties = array('server_addr','name','port','tls','pwd_encryption','binddn','bindpw','basedn','user_attr','mail_attr','ldap_to_drupal_user','testing_drupal_username','module_data');
    if (is_array($params)) {
      foreach (array_intersect($params,$constructor_properties) as $prop) {
        if (isset($params[$prop])) {
          $this->$prop = $params[$prop];
        }
      }  
    }
    elseif (is_numeric($params)) {
      $this->sid = $params;
      $record = db_fetch_object(db_query("SELECT * FROM {ldap_api_servers} WHERE sid = %d", $this->sid));
      foreach ($constructor_properties as $prop) {
        if (isset($record->$prop)) {
          $this->$prop = $record->$prop;
        }
      }  
    }
    
    if ($this->basedn) {
    // dpm('explode');  dpm(explode("\r\n", $this->basedn));
      $this->basedns = serialize(explode("\r\n", $this->basedn));
    }
  }

  function __destruct() {
    // should break any connection or binding
  }


  function connect($dn = NULL, $pass = NULL) {
    $this->disconnect();
    $dn = ($dn != NULL) ? $dn : $this->binddn;
    $pass = ($pass != NULL) ? $pass :  ldap_api_decrypt($this->bindpw, $this->pwd_encryption);
    return $this->connectAndBind($dn, $pass);
  }

  function initConnection() {
    if (!$con = ldap_connect($this->server_addr, $this->port)) {
      watchdog('ldap', 'LDAP Connect failure to '. $this->server_addr .':'. $this->port);
      return;
    }

    ldap_set_option($con, LDAP_OPT_PROTOCOL_VERSION, 3);
    ldap_set_option($con, LDAP_OPT_REFERRALS, 0);
    if ($this->tls) {
      ldap_get_option($con, LDAP_OPT_PROTOCOL_VERSION, $vers);
      if ($vers == -1) {
        watchdog('ldap', 'Could not get LDAP protocol version.');
        return;
      }
      if ($vers != 3) {
        watchdog('ldap', 'Could not start TLS, only supported by LDAP v3.');
        return;
      }
      else if (!function_exists('ldap_start_tls')) {
        watchdog('ldap', 'Could not start TLS. It does not seem to be supported by this PHP setup.');
        return;
      }
      else if (!ldap_start_tls($con)) {
        watchdog('ldap', t("Could not start TLS. (Error %errno: %error).", array('%errno' => ldap_errno($con), '%error' => ldap_error($con))));
        return;
      }
    }
    $this->connection = $con;
  }

  function connectAndBind($dn, $pass) {
    $this->initConnection();
    $con = $this->connection;
    if (!$this->bind($dn, $pass)) {
      watchdog('ldap', 'LDAP Bind failure for user %user. Error %errno: %error', array('%user' => $dn, '%errno' => ldap_errno($con), '%error' => ldap_error($con)));
      return NULL;
    }

    return $con;
  }

  function bind($dn, $pass) {
   // ob_start();
  //  set_error_handler(array('LDAPInterface', 'void_error_handler'));
    $ret = ldap_bind($this->connection, $dn, $pass);
  //  restore_error_handler();

   // ob_end_clean();

    return $ret;
  }

  function disconnect() {
    if ($this->connection) {
      ldap_unbind($this->connection);
      $this->connection = NULL;
    }
  }

  /**
 * get_ldap_user_entry_from_drupal_username
 *
 * @param $drupal_username
 *
 * @return
 *   If no matching user found in LDAP: FALSE 
 *   If match found: an associative array of LDAP User object properties
 */
  
  public function get_ldap_user_entry_from_drupal_username($drupal_username) {
  
    if ($ldap_server->ldap_to_drupal_user) {
      $name = $drupal_username;
      drupal_eval($ldap_server->ldap_to_drupal_user);
      $drupal_username = $name;
    }
    
    foreach (unserialize($this->basedns) as $base_dn) {
     $filter = $this->user_attr .'='. $drupal_username;
     $result = $this->search($base_dn, $filter);
     if (!$result)
       continue;
     
     // Must find exactly one user for authentication to.
     if ($result['count'] != 1) {
       watchdog('ldap', "Error: %num_matches users found with $%filter under %base_dn.", array('%num_matches' => $result['count'], '%filter' => $filter, '%base_dn' => $base_dn), WATCHDOG_ERROR);
       continue;
     }
     $match = $result[0];
 
     // These lines serve to fix the attribute name in case a
     // naughty server (i.e.: MS Active Directory) is messing the
     // characters' case.
     // This was contributed by Dan "Gribnif" Wilga, and described
     // here: http://drupal.org/node/87833
     if (!isset($match[$this->user_attr][0])) {
       $username_attr = drupal_strtolower($this->user_attr);
       if (!isset($match[$username_attr][0]))
         continue;
     }
     // Finally, we must filter out results with spaces added before
     // or after, which are considered OK by LDAP but are no good for us
     // We allow lettercase independence, as requested by Marc Galera
     // on http://drupal.org/node/97728
     //
     // Some setups have multiple $name_attr per entry, as pointed out by
     // Clarence "sparr" Risher on http://drupal.org/node/102008, so we
     // loop through all possible options.
     foreach ($match[$username_attr] as $value) {
       if (drupal_strtolower(trim($value)) == drupal_strtolower($drupal_username))
         return $match;
     }
     
    }
  

  }
  
  function search($base_dn, $filter, $attributes = array()) {
    $ret = array();
    // For the AD the '\,' should be replaced by the '\\,' in the search filter.
    $filter = preg_replace('/\\\,/', '\\\\\,', $filter);
   // set_error_handler(array('ldap_server', 'void_error_handler'));
    $x = ldap_search($this->connection, $base_dn, $filter, $attributes, 0, 50);
   // restore_error_handler();

    if ($x && ldap_count_entries($this->connection, $x)) {
      $ret = ldap_get_entries($this->connection, $x);
    }
   // print "<hr/>$ret"; print_r($ret);
   // die;
    return $ret;
  }

  // WARNING! WARNING! WARNING!
  // This function returns its entries with lowercase attribute names.
  // Don't blame me, blame PHP's own ldap_get_entries()
  function retrieveAttributes($dn) {
    set_error_handler(array('ldap_server', 'void_error_handler'));
    $result = ldap_read($this->connection, $dn, 'objectClass=*');
    $entries = ldap_get_entries($this->connection, $result);
    restore_error_handler();

    return call_user_func($this->attr_filter, $this->sid, $entries[0]);
  }

  function retrieveAttribute($dn, $attrname) {
    $entries = $this->retrieveAttributes($dn);
    return isset($entries[strtolower($attrname)]) ? $entries[strtolower($attrname)][0] : NULL;
  }

  function retrieveMultiAttribute($dn, $attrname) {
    $entries = $this->retrieveAttributes($dn);

    $result = array();
    $retrieved = $entries[strtolower($attrname)];
    $retrieved = $retrieved ? $retrieved : array();
    foreach ($retrieved as $key => $value) {
      if ($key !== 'count') {
        $result[] = $value;
      }
    }
    return $result;
  }

  function writeAttributes($dn, $attributes) {
    foreach ($attributes as $key => $cur_val) {
      if ($cur_val == '') {
        unset($attributes[$key]);
        $old_value = $this->retrieveAttribute($dn, $key);
        if (isset($old_value)) {
          ldap_mod_del($this->connection, $dn, array($key => $old_value));
        }
      }
      if (is_array ($cur_val)) {
        foreach ($cur_val as $mv_key => $mv_cur_val) {
          if ($mv_cur_val == '') {
            unset($attributes[$key][$mv_key]);
          }
          else {
            $attributes[$key][$mv_key] = $mv_cur_val;
          }
        }
      }
    }

    ldap_modify($this->connection, $dn, $attributes);
  }

  function create_entry($dn, $attributes) {
    set_error_handler(array('ldap_server', 'void_error_handler'));
    $ret = ldap_add($this->connection, $dn, $attributes);
    restore_error_handler();

    return $ret;
  }

  function rename_entry($dn, $newrdn, $newparent, $deleteoldrdn) {
    set_error_handler(array('ldap_server', 'void_error_handler'));
    $ret = ldap_rename($this->connection, $dn, $newrdn, $newparent, $deleteoldrdn);
    restore_error_handler();

    return $ret;
  }

  function delete_entry($dn) {
    set_error_handler(array('ldap_server', 'void_error_handler'));
    $ret = ldap_delete($this->connection, $dn);
    restore_error_handler();

    return $ret;
  }

  // This function is used by other modules to delete attributes once they are
  // moved to profiles cause ldap_mod_del does not delete facsimileTelephoneNumber if
  // attribute value to delete is passed to the function.
  // OpenLDAP as per RFC 2252 doesn't have equality matching for facsimileTelephoneNumber
  // http://bugs.php.net/bug.php?id=7168
  function deleteAttribute($dn, $attribute) {
    ldap_mod_del($this->connection, $dn, array($attribute => array()));
  }

  static function void_error_handler($p1, $p2, $p3, $p4, $p5) {
  }
  
}
